/*
 * sepgsql/hooks.h
 *
 * Header of SE-PostgreSQL Hooks
 *
 * Copyright (C) 2006-2010, NEC Corporation
 *                          KaiGai Kohei <kaigai@ak.jp.nec.com>
 */
#ifndef SEPGSQL_HOOKS_H
#define SEPGSQL_HOOKS_H

#include "executor/tuptable.h"
#include "nodes/plannodes.h"
#include "nodes/relation.h"
#include "storage/fd.h"
#include "utils/acl.h"
#include "utils/rel.h"
#include "utils/snapshot.h"

/*
 * misc.c
 */
extern char *sepgsql_get_client_label(void);
extern char *sepgsql_set_client_label(char *new_label);
extern void sepgsql_post_bootstraping(void);
extern void sepgsql_initialize(void);
extern bool sepgsql_worker_needed(void);
extern void sepgsql_worker_main(void);

extern Datum sepgsql_getcon(PG_FUNCTION_ARGS);

/*
 * database.c
 */
extern Oid  sepgsql_database_create(const char *datName, Oid templateOid);
extern void sepgsql_database_alter(Oid databaseOid);
extern Oid  sepgsql_database_relabel(Oid databaseOid, char *new_label);
extern void sepgsql_database_drop(Oid databaseOid, bool cascade);
extern void sepgsql_database_grant(Oid databaseOid);
extern void sepgsql_database_comment(Oid databaseOid);
extern void sepgsql_database_connect(Oid databaseOid);
extern void sepgsql_database_reindex(Oid databaseOid);
extern void sepgsql_database_getattr(Oid databaseOid);

/*
 * schema.c
 */
extern Oid  sepgsql_schema_create(const char *nspName, bool is_temp);
extern void sepgsql_schema_alter(Oid namespaceOid);
extern Oid  sepgsql_schema_relabel(Oid namespaceOid, char *new_label);
extern void sepgsql_schema_drop(Oid namespaceOid, bool cascade);
extern void sepgsql_schema_grant(Oid namespaceOid);
extern bool sepgsql_schema_search(Oid namespaceOid, bool abort);
extern void sepgsql_schema_comment(Oid namespaceOid);

/*
 * relation.c
 */
extern bool sepgsql_relation_perms(Oid relOid, AclMode aclmask,
								   Bitmapset *selectedCols,
								   Bitmapset *modifiedCols, bool abort);
extern Oid *sepgsql_relation_create(const char *relName,
									char relkind,
									TupleDesc tupDesc,
									Oid namespaceId,
									List *supOids,
									bool createAs);
extern void sepgsql_relation_alter(Oid relationOid);
extern void sepgsql_relation_alter_schema(Oid relationOid, Oid newSchema);
extern void sepgsql_relation_alter_rename(Oid relationOid,
										  const char *newName);
extern void sepgsql_relation_alter_inherit(Oid childOid, Oid parentOid);
extern Oid  sepgsql_relation_relabel(Oid relationOid, char *new_label);
extern void sepgsql_relation_drop(Oid relationOid, bool cascade);
extern void sepgsql_relation_getattr(Oid relationOid);
extern void sepgsql_relation_grant(Oid relationOid);
extern void sepgsql_relation_comment(Oid relationOid);
extern bool sepgsql_relation_cluster(Oid relationOid, bool abort);
extern void sepgsql_relation_truncate(Relation rel);
extern void sepgsql_relation_lock(Relation rel);
extern void sepgsql_relation_reindex(Oid relOid);
extern void sepgsql_view_replace(Oid viewOid);
extern void sepgsql_index_create(Oid relationOid, Oid namespaceOid);
extern void sepgsql_index_reindex(Oid indexOid);
extern void sepgsql_sequence_get_value(Oid sequenceOid);
extern void sepgsql_sequence_next_value(Oid sequenceOid);
extern void sepgsql_sequence_set_value(Oid sequenceOid);
extern void sepgsql_rule_create(Oid relationOid,
								const char *ruleName);
extern void sepgsql_rule_drop(Oid relationOid,
							  const char *ruleName, bool cascade);
extern void sepgsql_rule_comment(Oid relationOid, const char *ruleName);
extern void sepgsql_trigger_create(Oid relationOid, const char *triggerName,
								   Oid constrrelid, Oid funcOid);
extern void sepgsql_trigger_alter(Oid relOid, const char *tgName);
extern void sepgsql_trigger_drop(Oid relOid, const char *tgName, bool cascade);
extern void sepgsql_trigger_comment(Oid relOid, const char *tgName);
extern void sepgsql_constraint_comment(Oid relOid, const char *constName);

/*
 * attribute.c
 */
extern Oid  sepgsql_attribute_create(Oid relOid, const char *attName);
extern void sepgsql_attribute_alter(Oid relOid, const char *attName);
extern Oid  sepgsql_attribute_relabel(Oid relOid, const char *attName,
									  char *new_label);
extern void sepgsql_attribute_drop(Oid relOid, const char *attName, bool cascade);
extern void sepgsql_attribute_grant(Oid relOid, AttrNumber attnum);
extern void sepgsql_attribute_comment(Oid relOid, AttrNumber attnum);

/*
 * proc.c
 */
extern Oid  sepgsql_proc_create(const char *proName, Oid replaced,
								Oid namespaceOid, Oid langageOid);
extern void sepgsql_proc_alter(Oid procOid);
extern void sepgsql_proc_alter_rename(Oid procOid, const char *newName);
extern void sepgsql_proc_alter_schema(Oid procOid, Oid newSchema);
extern Oid  sepgsql_proc_relabel(Oid procOid, char *new_label);
extern void sepgsql_proc_drop(Oid procOid, bool cascade);
extern void sepgsql_proc_grant(Oid procOid);
extern void sepgsql_proc_comment(Oid procOid);
extern void sepgsql_proc_execute(Oid procOid);
extern bool sepgsql_proc_be_inlined(HeapTuple protup);
extern char *sepgsql_proc_domtrans(HeapTuple protup, MemoryContext mcxt);
extern Oid  sepgsql_aggregate_create(const char *aggName, Oid namespaceId,
									 Oid transFunc, Oid finalFunc);
extern void sepgsql_aggregate_execute(Oid aggOid);

/*
 * type.c
 */
extern Oid  sepgsql_type_create(const char *typeName, Oid replaced,
								Oid namespaceId, char typeType,
								Oid inputFunc, Oid outputFunc,
								Oid recvFunc, Oid sendFunc,
								Oid modinFunc, Oid modoutFunc,
								Oid analyzeFunc);
extern void sepgsql_type_alter(Oid typeOid);
extern void sepgsql_type_alter_rename(Oid typeOid, const char *newName);
extern void sepgsql_type_alter_schema(Oid typeOid, Oid newSchema);
extern Oid  sepgsql_type_relabel(Oid typeOid, char *newLabel);
extern void sepgsql_type_drop(Oid typeOid, bool cascade);
extern void sepgsql_type_comment(Oid typeOid);
extern Oid  sepgsql_cast_create(Oid sourceTypeOid, Oid targetTypeOid,
								char castMethod, Oid castFuncOid);
extern void sepgsql_cast_drop(Oid srcTypeOid, Oid dstTypeOid, bool cascade);
extern void sepgsql_cast_comment(Oid srcTypeOid, Oid dstTypeOid);

/*
 * tablespace.h
 */
extern Oid  sepgsql_tablespace_create(const char *tablespaceName);
extern void sepgsql_tablespace_alter(Oid tablespaceOid);
extern Oid  sepgsql_tablespace_relabel(Oid tablespaceOid, char *newLabel);
extern void sepgsql_tablespace_drop(Oid tablespaceOid, bool cascade);
extern void sepgsql_tablespace_grant(Oid tablespaceOid);
extern void sepgsql_tablespace_getattr(Oid tablespaceOid);
extern void sepgsql_tablespace_comment(Oid tablespaceOid);

/*
 * operator.h
 */
extern Oid  sepgsql_operator_create(const char *operName, Oid replaced,
									Oid namespaceId,
									Oid codeFn, Oid restrictFn, Oid joinFn,
									Oid commutatorOp, Oid negatorOp);
extern void sepgsql_operator_alter(Oid operOid);
extern Oid  sepgsql_operator_relabel(Oid operOid, char *newLabel);
extern void sepgsql_operator_drop(Oid operOid, bool cascade);
extern void sepgsql_operator_comment(Oid operOid);

extern Oid  sepgsql_opclass_create(const char *opcName, Oid namespaceId,
								   Oid typeOid, Oid opfamilyOid, Oid storageOid);
extern void sepgsql_opclass_alter(Oid opcOid);
extern void sepgsql_opclass_alter_rename(Oid opcOid, const char *newName);
extern void sepgsql_opclass_drop(Oid opcOid, bool cascade);
extern void sepgsql_opclass_comment(Oid opcOid);

extern Oid  sepgsql_opfamily_create(const char *opfName,
									Oid namespaceId, Oid amOid);
extern void sepgsql_opfamily_alter(Oid opfOid, bool isDrop, Oid amOid,
								   List *operators, List *procedures);
extern void sepgsql_opfamily_alter_rename(Oid opfOid, const char *newName);
extern void sepgsql_opfamily_alter_owner(Oid opfOid, Oid newOwner);
extern void sepgsql_opfamily_drop(Oid opfOid, bool cascade);
extern void sepgsql_opfamily_comment(Oid opfOid);

/*
 * role.c
 */
extern Oid  sepgsql_role_create(const char *roleName);
extern void sepgsql_role_alter(Oid roleOid);
extern Oid  sepgsql_role_relabel(Oid roleOid, char *newLabel);
extern void sepgsql_role_drop(Oid roleOid, bool cascade);
extern void sepgsql_role_grant(Oid roleOid, bool is_grant, List *memberIds);
extern void sepgsql_role_comment(Oid roleOid);

/*
 * blob.c
 */
extern Oid  sepgsql_largeobject_create(Oid loid);
extern void sepgsql_largeobject_alter(Oid loid);
extern Oid  sepgsql_largeobject_relabel(Oid loid, char *newLabel);
extern void sepgsql_largeobject_drop(Oid loid, bool cascade);
extern void sepgsql_largeobject_read(Oid loid, Snapshot snapshot);
extern void sepgsql_largeobject_write(Oid loid, Snapshot snapshot);
extern Oid  sepgsql_largeobject_import(Oid loid, const char *filename);
extern void sepgsql_largeobject_export(Oid loid, Snapshot snapshot,
									   const char *filename);
extern void sepgsql_largeobject_grant(Oid loid);
extern void sepgsql_largeobject_comment(Oid loid);

/*
 * conversion.c
 */
extern Oid  sepgsql_conversion_create(const char *convName,
									  Oid namespaceId, Oid conversionFunc);
extern void sepgsql_conversion_alter(Oid convOid);
extern void sepgsql_conversion_alter_rename(Oid convOid, const char *newName);
extern void sepgsql_conversion_drop(Oid convOid, bool cascade);
extern void sepgsql_conversion_comment(Oid convOid);

/*
 * tsearch.c
 */
extern Oid  sepgsql_ts_config_create(const char *confName, Oid namespaceId);
extern void sepgsql_ts_config_alter(Oid confOid);
extern void sepgsql_ts_config_alter_rename(Oid confOid, const char *newName);
extern void sepgsql_ts_config_drop(Oid confOid, bool cascade);
extern void sepgsql_ts_config_comment(Oid confOid);
extern Oid  sepgsql_ts_dict_create(const char *dictName, Oid namespaceId);
extern void sepgsql_ts_dict_alter(Oid dictOid);
extern void sepgsql_ts_dict_alter_rename(Oid dictOid, const char *newName);
extern void sepgsql_ts_dict_drop(Oid dictOid, bool cascade);
extern void sepgsql_ts_dict_comment(Oid dictOid);
extern Oid  sepgsql_ts_parser_create(const char *parseName, Oid namespaceId,
									 Oid startFunc, Oid tokenFunc, Oid endFunc,
									 Oid headlineFunc, Oid lextypeFunc);
extern void sepgsql_ts_parser_alter_rename(Oid parseOid, const char *newName);
extern void sepgsql_ts_parser_drop(Oid parseOid, bool cascade);
extern void sepgsql_ts_parser_comment(Oid parseOid);
extern Oid  sepgsql_ts_template_create(const char *templateName,
									   Oid namespaceId,
									   Oid initFunc, Oid lexizeFunc);
extern void sepgsql_ts_template_alter_rename(Oid templateOid,
											 const char *newName);
extern void sepgsql_ts_template_drop(Oid templateOid, bool cascade);
extern void sepgsql_ts_template_comment(Oid templateOid);

/*
 * fdw.c
 */
extern Oid  sepgsql_fdw_create(const char *fdwName, Oid validatorFunc);
extern void sepgsql_fdw_alter(Oid fdwOid, Oid newValidator);
extern void sepgsql_fdw_drop(Oid fdwOid, bool cascade);
extern void sepgsql_fdw_grant(Oid fdwOid);

extern Oid  sepgsql_fserver_create(const char *fservName, Oid fdwOid);
extern void sepgsql_fserver_alter(Oid fservOid);
extern void sepgsql_fserver_drop(Oid fservOid, bool cascade);
extern void sepgsql_fserver_grant(Oid fservOid);

extern Oid  sepgsql_user_mapping_create(Oid mappedRoleId, Oid fservOid);
extern void sepgsql_user_mapping_alter(Oid umapOid);
extern void sepgsql_user_mapping_drop(Oid umapOid, bool cascade);

extern void sepgsql_file_getattr(const char *filename);
extern void sepgsql_file_read(const char *filename);
extern void sepgsql_file_write(const char *filename, bool may_create);
extern void sepgsql_file_append(const char *filename);
extern void sepgsql_file_unlink(const char *filename);
extern void sepgsql_file_rename(const char *oldpath, const char *newpath);
extern void sepgsql_dir_list(const char *dirname);

/*
 * row-level access controls
 */
#define SEPGSQL_ROWLV_FILTER	1
#define SEPGSQL_ROWLV_ABORT		2
#define SEPGSQL_ROWLV_BYPASS	3

extern int  sepgsql_rowlv_get_mode(void);
extern int  sepgsql_rowlv_set_mode(int new_mode);
extern bool sepgsql_rowlv_quals(Relation rel, uint32 required,
								TupleTableSlot *slot, bool abort);
extern bool sepgsql_rowlv_copyto(Relation rel, HeapTuple tuple);
extern uint32 sepgsql_rowlv_permissions(RangeTblEntry *rte);

extern void sepgsql_tuple_insert(Relation rel, HeapTuple tuple);
extern void sepgsql_tuple_update(Relation rel, ItemPointer otid, HeapTuple newtup);

#endif	/* SEPGSQL_HOOKS_H */
